/********************************************************************
   T/SBDAS-7890/8420 Demo Utility for C3x DSP. rev.1A
   (C) MicroLAB Systems, 1999-2000


   Description:
   ------------
	1. This demo writes SINEs to DAC-0..7 channels
	   in ASYNCHRONOUS mode.
    2. DSP on-chip timer TM-0 is used for sampling packet frequency.
	3. DSP on-chip timer TM-1 is used for scan frequency generation.

   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-3x (except TORNADO-32L), TORNADO-P3x 
	and TORNADO-E3x	boards and actually differs in PIOX/SIOX reset control 
	and	initialization of C3x DSP on-chip timer in order to meet DSP cycle 
	time. This demo does not utilize any host PC ISA/PCI resources for 
	TORNADO-3x/P3x boards and can run in stand-alone mode. MIRAGE JTAG 
	emulator and TI CCS tools are required to recompile this demo and to 
	upload code to the DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and is provided as it is 
        without any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration jumpers:
          - no interrupt from T/SBDAS-7890/8420
	  - on-board 10MHz synchro-clock is used
	  - ASYNCHRONOUS mode is selected
	  - SIOX-Bus Device ID is '000'
	  - C3x SIO-0 port is used 

***********************************************************************/

#define	_E3X_		1
#define	_T3X_		2
#define	_TP3X_		3

#define	_C3X_SIO0_	0
#define	_C3X_SIO1_	1

//~~~~~~~~~ TORNADO and C3x SIO port selectors for compilation ~~~~~~~
//!!!! modify this def in order to compile for specific TORNADO platform
#define	_TORNADO_	_T3X_

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define	_E3X_SELECTED	(_TORNADO_==_E3X_)
#define	_T3X_SELECTED	(_TORNADO_==_T3X_)
#define	_TP3X_SELECTED	(_TORNADO_==_TP3X_)


#include	"float.h"
#include	"math.h"
#include	"c3x_def.h"
#include	"s7890_d.h"

//------- macros for SIO-0 port access ----------------------
#define	_WAIT_SIO_TRDY		while(!((*C3X_SIO0_GC) & 0x2))
#define	_WAIT_SIO_RRDY		while(!((*C3X_SIO0_GC) & 0x1))
#define	_WRITE_SIO_DATA		(*C3X_SIO0_XD)
#define _READ_SIO_DATA		(*C3X_SIO0_RD)


//------ TORNADO platform specific fucntions --------------------
#if(_E3X_SELECTED)
#define _REMOVE_SIOX_RESET	TE3X_set_pxsx_run(0x3)
#include	"te3x_dsp.h"
#endif

#if(_T3X_SELECTED)
// SIOX reset control is not available for T3x boards.
// In case T33 board is installed, disable PIOX/SIOX reset control via
// host PC interface.
#define _REMOVE_SIOX_RESET	;
#include	"t3x_dsp.h"
#endif

#if(_TP3X_SELECTED)
#define _REMOVE_SIOX_RESET	TP3X_set_pxsx_run(0x7)
#include	"tp3x_dsp.h"
#endif
//--------------------------------------------------------------


#define	SXBUS_DEV_ID		0




	//--- writes DATA to DAC channel and set ADC channel
void set_cdw(dac_chan, dac_data,adc_chan)
unsigned dac_data, dac_chan, adc_chan;
{
		//-- wait for SIO-0 transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send data write command
	_WRITE_SIO_DATA =(   (SXBUS_DEV_ID << TSBDAS_7890_CDW_DEV_ID_SHIFT)
					   | (dac_data & TSBDAS_7890_CDW_DAC_DATA_BITMASK)
					   | ((dac_chan & TSBDAS_7890_CDW_DAC_CHAN_BITMASK) << TSBDAS_7890_CDW_DAC_CHAN_SHIFT)
					   | ((adc_chan & TSBDAS_7890_CDW_ADC_CHAN_BITMASK) << TSBDAS_7890_CDW_ADC_CHAN_SHIFT)
			 		 )  ;
}

	//--- reads ADC DATA
void get_adc_data( adc_chan, adc_data)
unsigned *adc_chan, *adc_data;
{ unsigned rdr;

		//-- wait for receiver is ready and read data
	_WAIT_SIO_RRDY;
	rdr=_READ_SIO_DATA;
    *adc_chan = (rdr >> TSBDAS_7890_ADCODW_ADC_CHAN_SHIFT) & TSBDAS_7890_ADCODW_ADC_CHAN_BITMASK;
    *adc_data = (((rdr & TSBDAS_7890_ADCODW_ADC_DATA_BITMASK)<<20) >>20);
}




//-- DSP Init procedure
// This demo automatically detects C33 DSP in order to correctly
// set sampling frequency via DSP on-chip timer(s), since
// C33 clock is 150 MHz instead of 60MHz for C30/C31.

unsigned	dsp_type;

#define	C33_DSP_TYPE			0x33
#define	C30_C31_DSP_TYPE		0x30

#define	_C33_DSP				(dsp_type==C33_DSP_TYPE)
#define	_C30_C31_DSP			(dsp_type==C30_C31_DSP_TYPE)

#define	_RAM_TEST_PATTERN1	0x12345678
#define	_RAM_TEST_PATTERN2	0x87654321

void init_cpu()
 {
	unsigned	*mem;

// perform detection of C33 DSP type by means of write/read to different
// DSP on-chip memory areas:
//	- C33 DSP is detected in case 0x807ffe and 0x807fff memory locations are valid
//	- C30/C31 DSP are assumed otherwise


	// assume this is C33 DSP:
	mem = (unsigned *) (C33_ONCHIP_RAM_EADDR-1);
	mem[0]	= _RAM_TEST_PATTERN1;
	mem[1]	= _RAM_TEST_PATTERN2;
	if(	mem[0]==_RAM_TEST_PATTERN1 && mem[1]==_RAM_TEST_PATTERN2)
		{
			// success!!! This is C33 DSP in TORNADO-33/TP33/E33 board
			dsp_type = C33_DSP_TYPE;
		}
		else
		{
			// assume C30/C31 DSP otherwise
			dsp_type = C30_C31_DSP_TYPE;
		}

	// This is C30/C31 DSP in E3x/T3x/TP3x
	// (all C30/C31/C33-based boards have identical value for P-BUS COntrol Register
	*C3X_PBUS_GC	= 0x700;

    C3X_INT_EDGE;		// set falling edge IRQs for C33-based boards
//	C3X_CACHE_ENABLE;
	return;
 }


void init_sio()
{
		//-- configure C3x SIO port
	*C3X_SIO0_GC 	= 	TSBDAS_7890_C3X_SIO_GCRG_DATA;
	*C3X_SIO0_XC 	= 	TSBDAS_7890_C3X_SIO_XCRG_DATA;
	*C3X_SIO0_RC 	= 	TSBDAS_7890_C3X_SIO_RCRG_DATA;
	*C3X_SIO0_GC 	|= 	TSBDAS_7890_C3X_SIO_GCRG_GO_MASK;

		//-- release SIOX reset control and enable DCM
	_REMOVE_SIOX_RESET;
}



	//-- configure TM-0 and TM-1 for sampling frequency and
	//   scan frequency generation

#define	_TM_CONTROL_RG_DATA			0x00000301  // C/P=clock, CLKSRC=internal, FUNC=timer
#define	_TM_CONTROL_RG_RUN			0x000000c0

#define _SAMPLING_FREQ				2000		// desired sampling freq (Hz)
#define _SCAN_FREQ					50000		// desired scan freq in (Hz)

#define _C30_C31_TIMER_SRCCLK_FREQ	15000000	// C30/C31 TM-0/1 internal src clk (Hz)
#define _C33_TIMER_SRCCLK_FREQ		37500000	// C33 TM-0/1 internal src clk (Hz)

void set_fs_fscan()
{
		//-- configure TM-0 for sampling frequency
	*C3X_TM0_GC 	= _TM_CONTROL_RG_DATA;	
	*C3X_TM0_CNT 	= 0;	
	*C3X_TM0_PER  	= ((_C33_DSP? _C33_TIMER_SRCCLK_FREQ:_C30_C31_TIMER_SRCCLK_FREQ)/(2*_SAMPLING_FREQ));	


		//-- configure TM-0 for scan frequency
	*C3X_TM1_GC  	= _TM_CONTROL_RG_DATA;	
	*C3X_TM1_CNT	= 0;	
	*C3X_TM1_PER	= ((_C33_DSP? _C33_TIMER_SRCCLK_FREQ:_C30_C31_TIMER_SRCCLK_FREQ)/(2*_SCAN_FREQ));	

		//-- now start TM-0 timer
	*C3X_TM0_GC |= _TM_CONTROL_RG_RUN;

}

	//-- generates sampling frequency via TIMER-0
void	wait_fs()
{
	//-- wait until TM0 output is '0'
    while(((*C3X_TM0_GC) & 0x800)!=0);

	//-- wait until TM0 output is '1'
    while(((*C3X_TM0_GC) & 0x800)==0);
}

	//-- generates scan frequency via TIMER-1
void	wait_fscan()
{
	//-- wait until TM1 output is '0'
    while(((*C3X_TM1_GC) & 0x800)!=0);

	//-- wait until TM1 output is '1'
    while(((*C3X_TM1_GC) & 0x800)==0);
}

	//-- ENABLE/DISABLE _FSAN() is used inside the
	//   sampling packet in order to synchronize FSCAN to the
	//   sampling event.
void	enable_fscan()
{
		//-- start timer
	*C3X_TM1_CNT = 0;	
	*C3X_TM1_GC |= _TM_CONTROL_RG_RUN;

}

void	disable_fscan()
{
		//-- start timer
	*C3X_TM1_GC &= ~_TM_CONTROL_RG_RUN;

}






#define	_DAC_CHANNELS	8
#define	_ADC_CHANNELS	8

#define	_EPOCH_LENGTH	1000

#define	TWO_PI		(2.0*3.1415926)

	//-- DACs data arrays
int dac_data[8][_EPOCH_LENGTH];

void main()
{  int i,n;

	init_cpu();
	init_sio();

		//-- fill in DAC arrays with SINUS
		//   These arrays will be loaded to DACs on run-time.
		//   Each DAC channel has its own amplitude and frequency.
	for(i=0;i<_EPOCH_LENGTH;i++)
       {   
		   dac_data[0][i]=  (int) ((float) (2000.0 * sin( 5.0*TWO_PI*((float)i)/((float) _EPOCH_LENGTH))));
	       dac_data[1][i]=  (int) ((float) (1500.0 * sin( 7.0*TWO_PI*((float)i)/((float) _EPOCH_LENGTH))));
	       dac_data[2][i]=  (int) ((float) (1000.0 * sin(10.0*TWO_PI*((float)i)/((float) _EPOCH_LENGTH))));
	       dac_data[3][i]=  (int) ((float) ( 750.0 * sin(15.0*TWO_PI*((float)i)/((float) _EPOCH_LENGTH))));
	       dac_data[4][i]=  (int) ((float) ( 500.0 * sin(20.0*TWO_PI*((float)i)/((float) _EPOCH_LENGTH))));
	       dac_data[5][i]=  (int) ((float) ( 400.0 * sin(30.0*TWO_PI*((float)i)/((float) _EPOCH_LENGTH))));
	       dac_data[6][i]=  (int) ((float) ( 300.0 * sin(40.0*TWO_PI*((float)i)/((float) _EPOCH_LENGTH))));
	       dac_data[7][i]=  (int) ((float) ( 200.0 * sin(50.0*TWO_PI*((float)i)/((float) _EPOCH_LENGTH))));
	   }

	set_fs_fscan();	// configure TM-0/1 for sampling/scan freq

        i=0;
	while (1)
	 {
		//-- wait for sampling freq event
	  wait_fs();	
	  enable_fscan();	// enable Fscan generator via TIMER-1
	  			// (used to synchronize Fscan to Fs and
				//  to exclude phase jitter)

          	//-- load DACs
          for(n=0;n<_DAC_CHANNELS;n++)
	    { 
          wait_fscan();
		  set_cdw(n,dac_data[n][i],0);
	    }

	  disable_fscan();	// disable Fscan generator via TIMER-1

      i++;
	  i %= _EPOCH_LENGTH;

	 }
}

