/********************************************************************
   T/SBDAS-7890/8420 Demo Utility for C6x. rev.1A
   (C) MicroLAB Systems, 1999-2000


   Description:
   ------------
	1. This demo writes SINEs to DAC-0..7 channels
	   in ASYNCHRONOUS mode.
        2. DSP on-chip timer TM-0 is used for sampling packet frequency.
	3. DSP on-chip timer TM-1 is used for scan frequency generation.

   TORNADO platform:
   -----------------
        This demo compiles for TORNADO-6x, TORNADO-P6x and TORNADO-E6x
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. MIRAGE JTAG emulator and TI CCS tools are
	required to recompile this demo and to upload code to the
	DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and is provided as it is 
        without any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration jumpers:
          - no interrupt from T/SBDAS-7890/8420
	  - on-board 10MHz synchro-clock is used
	  - ASYNCHRONOUS mode is selected
	  - SIOX-Bus Device ID is '000'
	  - C6x McBSP-0 port is used (change this via _C6X_MCBSP_ def)

***********************************************************************/

#define	_E6X_		1
#define	_T6X_		2
#define	_TP6X_		3

#define	_C6X_MCBSP0_	0
#define	_C6X_MCBSP1_	1

//~~~~~~~~~ TORNADO and C6x McBSP port selectors for compilation ~~~~~~~
//!!!! modify this def in order to compile for specific TORNADO platform
#define	_TORNADO_	_E6X_

//!!!! modify this def in order to compile for McBSP-0 or McBSP-1 port
#define	_C6X_MCBSP_	_C6X_MCBSP0_
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define	_E6X_SELECTED	(_TORNADO_==_E6X_)
#define	_T6X_SELECTED	(_TORNADO_==_T6X_)
#define	_TP6X_SELECTED	(_TORNADO_==_TP6X_)


#define	_C6X_MCBSP0_SELECTED	(_C6X_MCBSP_==_C6X_MCBSP0_)
#define	_C6X_MCBSP1_SELECTED	(_C6X_MCBSP_==_C6X_MCBSP1_)


#include	"math.h"
#include	"c6x_def.h"
#include	"s7890_d.h"

//------- macros for McBSP port access ----------------------
#if(_C6X_MCBSP0_SELECTED)	// define for C6x McBSP-0 port
#define	_WAIT_SIO_TRDY		while(!((*C6X_MCBSP0_SPCR) & 0x20000))
#define	_WAIT_SIO_RRDY		while(!((*C6X_MCBSP0_SPCR) & 0x2))
#define	_WRITE_SIO_DATA		(*C6X_MCBSP0_DXR)
#define _READ_SIO_DATA		(*C6X_MCBSP0_DRR)
#endif

#if(_C6X_MCBSP1_SELECTED)	// define for C6x McBSP-1 port
#define	_WAIT_SIO_TRDY		while(!((*C6X_MCBSP1_SPCR) & 0x20000))
#define	_WAIT_SIO_RRDY		while(!((*C6X_MCBSP1_SPCR) & 0x2))
#define	_WRITE_SIO_DATA		(*C6X_MCBSP1_DXR)
#define _READ_SIO_DATA		(*C6X_MCBSP1_DRR)
#endif


//------ TORNADO platform specific fucntions --------------------
#if(_E6X_SELECTED)
#define _REMOVE_SIOX_RESET	TE6X_set_pxsx_run(0x3)
#include	"te6x_dsp.h"
#endif

#if(_T6X_SELECTED)
#define _REMOVE_SIOX_RESET	T6X_set_pxsx_run(0x7)
#include	"t6x_dsp.h"
#endif

#if(_TP6X_SELECTED)
#define _REMOVE_SIOX_RESET	TP6X_set_pxsx_run(0x7)
#include	"tp6x_dsp.h"
#endif
//--------------------------------------------------------------


#define	SXBUS_DEV_ID		0




	//--- writes DATA to DAC channel and set ADC channel
void set_cdw(dac_chan, dac_data,adc_chan)
unsigned dac_data, dac_chan, adc_chan;
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send data write command
	_WRITE_SIO_DATA =(   (SXBUS_DEV_ID << TSBDAS_7890_CDW_DEV_ID_SHIFT)
			   | (dac_data & TSBDAS_7890_CDW_DAC_DATA_BITMASK)
			   | ((dac_chan & TSBDAS_7890_CDW_DAC_CHAN_BITMASK) << TSBDAS_7890_CDW_DAC_CHAN_SHIFT)
			   | ((adc_chan & TSBDAS_7890_CDW_ADC_CHAN_BITMASK) << TSBDAS_7890_CDW_ADC_CHAN_SHIFT)
			 )  ;
}

	//--- reads ADC DATA
void get_adc_data( adc_chan, adc_data)
unsigned *adc_chan, *adc_data;
{ unsigned rdr;

		//-- wait for receiver is ready and read data
	_WAIT_SIO_RRDY;
	rdr=_READ_SIO_DATA;
        *adc_chan = (rdr >> TSBDAS_7890_ADCODW_ADC_CHAN_SHIFT) & TSBDAS_7890_ADCODW_ADC_CHAN_BITMASK;
        *adc_data = (((rdr & TSBDAS_7890_ADCODW_ADC_DATA_BITMASK)<<20) >>20);
}






void init_cpu()
 {
	 // assume that C6x EMIF is initialized as default on reset
 }

void init_sio()
{
		//-- configure C6x McBSP port

#if(_C6X_MCBSP0_SELECTED)	// define for C6x McBSP-0 port
	*C6X_MCBSP0_SPCR = TSBDAS_7890_C6X_MCBSP_SPCR_DATA;
	*C6X_MCBSP0_PCR  = TSBDAS_7890_C6X_MCBSP_PCR_DATA;
	*C6X_MCBSP0_XCR  = TSBDAS_7890_C6X_MCBSP_XCR_DATA;
	*C6X_MCBSP0_RCR  = TSBDAS_7890_C6X_MCBSP_RCR_DATA;
	*C6X_MCBSP0_SRGR = TSBDAS_7890_C6X_MCBSP_SRGR_DATA;
	*C6X_MCBSP0_SPCR |= TSBDAS_7890_C6X_MCBSP_SPCR_GO_MASK;
#endif

#if(_C6X_MCBSP1_SELECTED)	// define for C6x McBSP-1 port
	*C6X_MCBSP1_SPCR = TSBDAS_7890_C6X_MCBSP_SPCR_DATA;
	*C6X_MCBSP1_PCR  = TSBDAS_7890_C6X_MCBSP_PCR_DATA;
	*C6X_MCBSP1_XCR  = TSBDAS_7890_C6X_MCBSP_XCR_DATA;
	*C6X_MCBSP1_RCR  = TSBDAS_7890_C6X_MCBSP_RCR_DATA;
	*C6X_MCBSP1_SRGR = TSBDAS_7890_C6X_MCBSP_SRGR_DATA;
	*C6X_MCBSP1_SPCR |= TSBDAS_7890_C6X_MCBSP_SPCR_GO_MASK;
#endif

		//-- release SIOX reset control and enable DCM
	_REMOVE_SIOX_RESET;
}


	//-- configure TM-0 and TM-1 for sampling frequency and
	//   scan frequency generation

#define	_TM_CONTROL_RG_DATA	0x00000301      // C/P=clock, CLKSRC=internal, FUNC=timer
#define	_TM_CONTROL_RG_RUN	0x000000c0

#define _SAMPLING_FREQ		2000		// sampling freq (Hz)
#define _SCAN_FREQ		80000		// scan freq in (Hz)

#define _C6X_TIMER_SRCCLK_FREQ	50000000	// TM-0/1 internal src clk (Hz)
						// (for C6201 @ 200MHz)

void set_fs_fscan()
{
		//-- configure TM-0 for sampling frequency
	*C6X_TM0_CONTROL = _TM_CONTROL_RG_DATA;	
	*C6X_TM0_COUNTER = 0;	
	*C6X_TM0_PERIOD  = (_C6X_TIMER_SRCCLK_FREQ/(2*_SAMPLING_FREQ));	


		//-- configure TM-0 for scan frequency
	*C6X_TM1_CONTROL = _TM_CONTROL_RG_DATA;	
	*C6X_TM1_COUNTER = 0;	
	*C6X_TM1_PERIOD  = (_C6X_TIMER_SRCCLK_FREQ/(2*_SCAN_FREQ));	

		//-- now start TM-0 timer
	*C6X_TM0_CONTROL |= _TM_CONTROL_RG_RUN;

}

	//-- generates sampling frequency via TIMER-0
void	wait_fs()
{
	//-- wait for TM0 output is '0'
    while(((*C6X_TM0_CONTROL) & 0x800)!=0);

	//-- wait for TM1 output is '1'
    while(((*C6X_TM0_CONTROL) & 0x800)==0);
}

	//-- generates scan frequency via TIMER-1
void	wait_fscan()
{
	//-- wait for TM1 output is '0'
    while(((*C6X_TM1_CONTROL) & 0x800)!=0);

	//-- wait for TM0 output is '1'
    while(((*C6X_TM1_CONTROL) & 0x800)==0);
}

	//-- ENABLE/DISABLE _FSAN() is used inside the
	//   sampling packet in order to synchronize FSCAN to the
	//   sampling event.
void	enable_fscan()
{
		//-- start timer
	*C6X_TM1_COUNTER = 0;	
	*C6X_TM1_CONTROL |= _TM_CONTROL_RG_RUN;

}

void	disable_fscan()
{
		//-- start timer
	*C6X_TM1_CONTROL &= ~_TM_CONTROL_RG_RUN;

}






#define	_DAC_CHANNELS	8
#define	_ADC_CHANNELS	8

#define	_EPOCH_LENGTH	1000

#define	TWO_PI		(2.0*3.1415926)

	//-- DACs data arrays
int dac_data[_DAC_CHANNELS][_EPOCH_LENGTH];

void main()
{  int i,n;

	init_cpu();
	init_sio();

		//-- fill in DAC arrays with SINUS
		//   These arrays will be loaded to DACs on run-time.
		//   Each DAC channel has its own amplitude and frequency.
	for(i=0;i<_EPOCH_LENGTH;i++)
           {   dac_data[0][i]=  (int) ((float) (2000.0 * sin( 5.0*TWO_PI*((float)i)/((float) _EPOCH_LENGTH))));
	       dac_data[1][i]=  (int) ((float) (1500.0 * sin( 7.0*TWO_PI*((float)i)/((float) _EPOCH_LENGTH))));
	       dac_data[2][i]=  (int) ((float) (1000.0 * sin(10.0*TWO_PI*((float)i)/((float) _EPOCH_LENGTH))));
	       dac_data[3][i]=  (int) ((float) ( 750.0 * sin(15.0*TWO_PI*((float)i)/((float) _EPOCH_LENGTH))));
	       dac_data[4][i]=  (int) ((float) ( 500.0 * sin(20.0*TWO_PI*((float)i)/((float) _EPOCH_LENGTH))));
	       dac_data[5][i]=  (int) ((float) ( 400.0 * sin(30.0*TWO_PI*((float)i)/((float) _EPOCH_LENGTH))));
	       dac_data[6][i]=  (int) ((float) ( 300.0 * sin(40.0*TWO_PI*((float)i)/((float) _EPOCH_LENGTH))));
	       dac_data[7][i]=  (int) ((float) ( 200.0 * sin(50.0*TWO_PI*((float)i)/((float) _EPOCH_LENGTH))));
	   }

	set_fs_fscan();	// configure TM-0/1 for sampling/scan freq

        i=0;
	while (1)
	 {
		//-- wait for sampling freq event
	  wait_fs();	
	  enable_fscan();	// enable Fscan generator via TIMER-1
	  			// (used to synchronize Fscan to Fs and
				//  to exclude phase jitter)

          	//-- load DACs
          for(n=0;n<_DAC_CHANNELS;n++)
	    { set_cdw(n,dac_data[n][i],0);
              wait_fscan();
	    }

	  disable_fscan();	// disable Fscan generator via TIMER-1

          i++;
	  i %=_EPOCH_LENGTH;

	 }
}

