/***********************************************************************
 Analog I/O Demo for TORNADO-3x with T/SBDAS-AD8/DA8-100/12.
 Copyright (C) MicroLAB(tm) Systems Ltd, 1998

 demo description:
   This demo provides DAC/ADC signal acquisition.

 available demo options:
   0 : Output F0/2xF0/../8xF0 sine waves via DAC-0/DAC-1/../DAC-7
   1 : Continuous pass-through (ADC-0=>DAC-0,ADC-1=>DAC-1, .. ADC-7=>DAC-7)

*************************************************************************/



#include <malloc.h>
#include <stdio.h>
#include <io.h>
#include <dos.h>
#include <conio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include <mlu.h>
#include <ips_ut.h>
#include <t3x_h.h>
#include "t3_comm.i"

#define  SCAN_LEN_MIN   1L
#define  SCAN_LEN_MAX   200000L


#define  MCMND_FFT	  0x104L
#define  USER_CMD	  0x100L

long   scan_len=10000L,   // epoch length in samples
       kbd_cmnd,cmnd;       // amplitude

long	i,chan=8;
int	tmp,ex_status, fq_code=1, in_gain=0, out_gain=1; //,adc_scan_flag=0
char	s[80],ch;
int	dat_type=4; // def for LONG type (number of bytes per word)

MLU_TBDD TB;  // time bell descriptor


#define CLEAR_CURRENT_LINE  printf("\r%79s\r"," ");



struct MCD_DD  *MCD;     // MCD pointer

long   *D,               // master data buffer array
       MADC_SADDR,
       MDAC_SADDR;
long   SCAN_LEN_DEFAULT;


void	cycle_pause()
 { long i;

 	for(i=0;i<1000000;i++);
 }


inp_param()
{
      printf("number of scan channels: ");

      chan=(int)getlong(1L,8L,(long)chan,CR_ENABLE,ESC_MASK,&ex_status);

      if(ex_status==ESC_MASK)
       {aborted:
	  printf(" <aborted>\n");
	  return(1);
       }

      return(0);

}


/*****************************************************************************
// fill on-board SRAM with zero or sine waves with F0 and 2xF0 frequencies
*****************************************************************************/

#define FILL_SINE   1
#define FILL_ZERO   0

 int __fill_dac_sram(int fill_mode)
  {
    long count, start, q, n, n1, j, chan_count, w_len;
    unsigned pmap_old;

      pmap_old=T3X_SMP_RG_DATA;

      CLEAR_CURRENT_LINE;
      printf("filling SRAM ...\r");

      for (chan_count=0; chan_count<8; chan_count++)
       {
	start=MDAC_SADDR/4L;
  	j=start+(scan_len*chan_count);
	count=scan_len;
	n=0L;

	do
	  {
            T3X_set_smp_ptr(j*dat_type,&D);

	    w_len=(T3X_SMP_LEN/dat_type)-j%(T3X_SMP_LEN/dat_type);

            if(count>=w_len) q=w_len;
	      else	q=count;

            for(n1=0;n1<q;n1++)
	     {
	      if(fill_mode==FILL_SINE)
	        {
		 switch(chan_count)
		  {
		   case 0:
		     *D++ = (long) ((double) (2000.0 * sin(10.0*M_PI*((double)(n1+n))/((double) scan_len))));
		    break;
		   case 1:
		     *D++ = (long) ((double) (2000.0 * sin(15.0*M_PI*((double)(n1+n))/((double) scan_len))));
		    break;
		   case 2:
		     *D++ = (long) ((double) (2000.0 * sin(20.0*M_PI*((double)(n1+n))/((double) scan_len))));
		    break;
		   case 3:
		     *D++ = (long) ((double) (2000.0 * sin(30.0*M_PI*((double)(n1+n))/((double) scan_len))));
		    break;
		   case 4:
		     *D++ = (long) ((double) (2000.0 * sin(40.0*M_PI*((double)(n1+n))/((double) scan_len))));
		    break;
		   case 5:
		     *D++ = (long) ((double) (2000.0 * sin(60.0*M_PI*((double)(n1+n))/((double) scan_len))));
		    break;
		   case 6:
		     *D++ = (long) ((double) (2000.0 * sin(80.0*M_PI*((double)(n1+n))/((double) scan_len))));
		    break;
		   case 7:
		     *D++ = (long) ((double) (2000.0 * sin(120.0*M_PI*((double)(n1+n))/((double) scan_len))));
		    break;
		  }
	        }
	       else
	         *D++ = 0L;
	     }

            if(T3X_SB_ERR_DATA)
	      {
	        CLEAR_CURRENT_LINE;
                printf("error: SB error\n\n");
                return(1);
	      }
	    count-=q;
	    j+=q;
	    n+=q;
	  }
	while (count>0);
       }


     T3X_SET_SMP_RG(pmap_old);
     CLEAR_CURRENT_LINE;
     return(0);
  }



 int __read_sb_data(long sb_addr, long len, void *pc_ptr)
  {
    long *LPTR,j, count,n1,q, w_len;


            LPTR=(long *)pc_ptr;
	    j=sb_addr;
	    count=len;

	    do
	      {

                T3X_set_smp_ptr(j*dat_type,&D);
		w_len=(T3X_SMP_LEN/dat_type)-j%(T3X_SMP_LEN/dat_type);
                if(count>=w_len) q=w_len;
		    else	q=count;

                for(n1=0;n1<q;n1++, *LPTR++= *D++);

                if(T3X_SB_ERR_DATA)
	           {
		      CLEAR_CURRENT_LINE;
                     printf("error: SB error r\n\n");
           	     return(1);
	           }
		count-=q;
		j+=q;
	      }
	     while (count>0);
       return(0);

  }


 int __write_sb_data(long sb_addr, long len, void *pc_ptr)
  {
    long *LPTR,j, count,n1,q, w_len;


            LPTR=(long *)pc_ptr;
	    j=sb_addr;
	    count=len;

	    do
	      {

                T3X_set_smp_ptr(j*dat_type,&D);
		w_len=(T3X_SMP_LEN/dat_type)-j%(T3X_SMP_LEN/dat_type);
                if(count>=w_len) q=w_len;
		    else	q=count;

                for(n1=0;n1<q;n1++, *D++=*LPTR++ );

                if(T3X_SB_ERR_DATA)
	           {
		      CLEAR_CURRENT_LINE;
                     printf("error: SB error w\n\n");
           	     return(1);
	           }
		count-=q;
		j+=q;
	      }
	     while (count>0);
       return(0);

  }


#define TMP_BUF_LEN	0x8000L


 int __move_adc_to_dac(long buf_len)
  {
    long count, j, q, *TMP, jd;



      CLEAR_CURRENT_LINE;
      printf("moving SRAM data ...\r");


    if((TMP=(long *) halloc(TMP_BUF_LEN*4L,1))==NULL)

       { printf("error: out of memory\n\n");
	 return(1);
       }

                      /* copy ADC data to DAC buffer */

	    j=MADC_SADDR/4L;
	    jd=MDAC_SADDR/4L;
	    count=buf_len;

	    do
	      {
                if(count>=TMP_BUF_LEN) q=TMP_BUF_LEN;
		    else	q=count;
		if(   __read_sb_data(j,q,TMP)!=0
		   || __write_sb_data(jd,q,TMP)!=0
		  )
           	   {
                     hfree(TMP);return(1);
	           }
		count-=q;
		j+=q;
		jd+=q;
	      }
	     while (count>0);


    CLEAR_CURRENT_LINE;
    hfree(TMP);
    return(0);
  }



int do_cmd(int adc_scan_enable)
 {
  T3X_set_smp_ptr(MCD_SADDR,&MCD);

  MCD->cmnd		= cmnd;
  MCD->epoch_len	= scan_len;
  MCD->status		= MSTAT_ATTN;
  MCD->chan		= chan;

  T3X_SET_HM_RQ;  // set MASTER ATTN via interrupt

  printf(" T3x operation in progress. Press <SPACE> to terminate or <ESC> to abort...\r");

  do                      // wait for the ESC pressed or data ready
   {

     if(key_pressed())
       { switch (read_char())
	  { case KEY_ESC:
 	      exe_aborted:
	         CLEAR_CURRENT_LINE;
	         printf("<execution aborted, T3X reset>\n");
	      m_restart:
	         T3X_SET_MRESET;
		 T3X_CLR_SB_ERR;
	         return(1);

            case 0x20:  // issue ADC SCAN synch
                 if (adc_scan_enable)
		  {
	    	      { CLEAR_CURRENT_LINE;
		        printf("\rADC scanning...\r");
		        MCD->status=MSTAT_GO;
		      }
		  }
		  else
		  {
		     CLEAR_CURRENT_LINE;
		        MCD->status=MSTAT_ABORT;
		     printf("\r Operation is aborted...\r");
		  }
		 break;
            default:
		 break;
	  }
       }
   }
    while(MCD->status!=MSTAT_RDY);
    CLEAR_CURRENT_LINE;

   if (MCD->err!=MLU_OK)
     { CLEAR_CURRENT_LINE;
       printf("error: %ld\n", MCD->err);
       return (1);
     }
   return(0);
 }



main()

{ int r;
  char err_mess[100], dev_name[100];
  unsigned dev_id;


   printf("\nAnalog I/O Demo for TORNADO-3x with T/SBDAS-AD8/DA8-100/12. v.1.0\n");
   printf("(C) MicroLAB Systems Ltd, 1998\n\n");

   if(   (r=T3X_open(0x350))!=T3X_OK
      || (r=T3X_dev_id(&dev_id))!=T3X_OK
      || (r=T3X_dev_name(dev_name))!=T3X_OK
     )
     {  error:
          T3X_last_err_message(err_mess);
	  printf("error: %s\n", err_mess);
	  goto exit;
     }

   printf(" %s, s/n: #%x\n", dev_name, dev_id & 0xff);

   T3X_set_smp_ptr(MCD_SADDR,&MCD);
   T3X_CLR_SB_ERR;

   T3X_SET_MRESET;       // init T3x and validate the response code
   T3X_SET_SB_CCL_BYTE;

   if(inp_param()!=0) goto exit;

      MCD->m_id=0xffffffff;
      MCD->prg_id=0xffffffff;
      MCD->status=0xffffffff;
      MCD->err=0xffffffff;
      MCD->adc_data_saddr=0xffffffff;
      MCD->dac_data_saddr=0xffffffff;

   MLU_set_bell(1,&TB);  // set time control =1s
   T3X_SET_MGO;

   while (MCD->status!=MSTAT_RDY) // wait for DSP will init or timeout
     {
       if(MLU_check_bell(&TB)<0)
	   { printf("error: no reply from TORNADO-3x\n\n");
	     goto exit;
	   }
       if(T3X_SB_ERR_DATA!=0)
	   { printf("error: SB error  main\n\n");
 		while(!key_pressed());
	     goto exit;
	   }
     }


   if(   MCD->m_id!=T3X_M_ID       // validate MCD returned
      || MCD->prg_id!=T3X_PRG_ID
      || MCD->status!=MSTAT_RDY
      || MCD->err!=MERR_OK
      || MCD->adc_data_saddr<=0L
      || MCD->adc_data_saddr>=T3X_SRAM_LEN_MAX
      || MCD->dac_data_saddr<=0L
      || MCD->dac_data_saddr>=T3X_SRAM_LEN_MAX
     )
       { printf("error: invalid T3x initialization data\n");
	 goto exit;
       }

   MADC_SADDR = MCD->adc_data_saddr*(long)dat_type;
   MDAC_SADDR = MCD->dac_data_saddr*(long)dat_type;
   scan_len = SCAN_LEN_DEFAULT = MCD->epoch_len;

   __fill_dac_sram(FILL_ZERO);


while(1)
 {
   printf("\n commands : \n");
   printf("      0 : Output F0..NxF0 sine waves via DAC-1..DAC-N\n");
   printf("      1 : Continuous pass-through (ADC-1=>DAC-1..ADC-N=>DAC-N)\n");
   printf("    Esc : exit\n\n");

   printf("enter command: ");


kbd_loop:
   switch(read_char())
     {
        case '0':
                CLEAR_CURRENT_LINE;
		printf("Output F0/NxF0 sine waves via DAC-Ch#1/Ch#N >>>>\n");

		cmnd = MCMND_DAS_DAC_SCAN;
		scan_len=SCAN_LEN_DEFAULT;

		if (    __fill_dac_sram(FILL_SINE)!=0
                     ||  do_cmd(MLU_OFF)!=0
		     || __fill_dac_sram(FILL_ZERO)!=0
                   ) goto exit;
		break;

	case '1':

                CLEAR_CURRENT_LINE;
		printf("Continuous Pass-through Ch#1..Ch#N data (ADC->DAC) >>>>\n");
		scan_len=100;
		cmnd = MCMND_ADC_TO_DAC;

		if (    __fill_dac_sram(FILL_ZERO)!=0
		     ||	do_cmd(MLU_OFF)!=0
		   ) goto exit;
                break;


        case KEY_ESC:
	  printf(" <aborted>\n");
	  goto exit;

	default:
		beep_err();
		goto kbd_loop;
     }

 }
exit:
  T3X_SET_MRESET;
  printf("\n");

}
