/********************************************************************
   T/SBDAS-7890/8420 Demo Utility for C3x DSP. rev.1A
   (C) MicroLAB Systems, 1999-2000


   Description:
   ------------
	1. This demo provides eight pass-through AD/DA channels,
	   i.e. output of ADC[i] is loaded to DAC[i].
	2. T/SBDAS-7890/8420 must be configured in SYNCHRONOUS mode.
    3. DSP on-chip timer TM-0 is used for sampling frequency generation.


   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-3x (except TORNADO-32L), TORNADO-P3x 
	and TORNADO-E3x	boards and actually differs in PIOX/SIOX reset control 
	and	initialization of C3x DSP on-chip timer in order to meet DSP cycle 
	time. This demo does not utilize any host PC ISA/PCI resources for 
	TORNADO-3x/P3x boards and can run in stand-alone mode. MIRAGE JTAG 
	emulator and TI CCS tools are required to recompile this demo and to 
	upload code to the DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and is provided as it is 
    without any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration jumpers:
          - SIOX IRQ-0 must be set on T/SBDAS-7890/8420
	  - on-board 10MHz synchro-clock is used
	  - SYNCHRONOUS mode is selected
	  - SIOX-Bus Device ID is '000'
	  - C3x SIO-0 port is used 

***********************************************************************/

#define	_E3X_		1
#define	_T3X_		2
#define	_TP3X_		3

#define	_C3X_SIO0_	0
#define	_C3X_SIO1_	1

//~~~~~~~~~ TORNADO and C3x SIO port selectors for compilation ~~~~~~~
//!!!! modify this def in order to compile for specific TORNADO platform
#define	_TORNADO_	_T3X_

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define	_E3X_SELECTED	(_TORNADO_==_E3X_)
#define	_T3X_SELECTED	(_TORNADO_==_T3X_)
#define	_TP3X_SELECTED	(_TORNADO_==_TP3X_)


#include	"math.h"
#include	"c3x_def.h"
#include	"s7890_d.h"

//-- macros for SIO-0 port access
#define	_WRITE_SIO_DATA		(*C3X_SIO0_XD)
#define _READ_SIO_DATA		(*C3X_SIO0_RD)
#define _CLEAR_SIO_RINT_	{C3X_SR0_INT_CLEAR; C3X_SR0_INT_ENABLE;}
#define	_ENABLE_SIO_RINT_	C3X_SR0_INT_ENABLE


//------ TORNADO platform specific fucntions --------------------
#if(_E3X_SELECTED)
#define _REMOVE_SIOX_RESET	TE3X_set_pxsx_run(0x3)
#include	"te3x_dsp.h"
#endif

#if(_T3X_SELECTED)
// SIOX reset control is not available for T3x boards.
// In case T33 board is installed, disable PIOX/SIOX reset control via
// host PC interface.
#define _REMOVE_SIOX_RESET	;
#include	"t3x_dsp.h"
#endif

#if(_TP3X_SELECTED)
#define _REMOVE_SIOX_RESET	TP3X_set_pxsx_run(0x7)
#include	"tp3x_dsp.h"
#endif
//--------------------------------------------------------------


#define	SXBUS_DEV_ID		0



		/* X/R interrupt handler parameters	*/

int	*dac_data_start_ptr,
    *dac_get_ptr,
	dac_data_count,
    dac_data_count_max;

int	*adc_save_ptr,
	*adc_data_start_ptr,
	adc_data_count,
	adc_data_count_max ;



	//-- F-scan interrupt fm T/SBDAS-7890/8420 DCM.
	//   This IRQ is generated via SIOX IRQ-0 interrupt request.
	//   Writes COMMAND DATA WORD to DCM.

unsigned chan_count = 0, max_chan=0;

interrupt void c_int01()
{
	//   Create COMMAND DATA WORD.
	//   get DAC data fm particular subarray of DATA array,
	//   which is filled in with real-time ADC data.

	_WRITE_SIO_DATA =(   (SXBUS_DEV_ID << TSBDAS_7890_CDW_DEV_ID_SHIFT)
					   | (chan_count << TSBDAS_7890_CDW_DAC_CHAN_SHIFT)
					   | (chan_count << TSBDAS_7890_CDW_ADC_CHAN_SHIFT )
					   | ((chan_count>=max_chan) ?  TSBDAS_7890_CDW_EOS:TSBDAS_7890_CDW_NO_EOS )
					   | ((dac_get_ptr[dac_data_count_max * chan_count]) & TSBDAS_7890_CDW_DAC_DATA_BITMASK )
					 )  ;

	if( (++chan_count) > max_chan)
	   {
	     dac_get_ptr++;
	     chan_count=0;

	     if((++dac_data_count) >=dac_data_count_max)
	       {       
				dac_data_count=0;
	            dac_get_ptr=dac_data_start_ptr;
	       }
	   }

}


	//-- Receiver interrupt fm T/SBDAS-7890/8420 DCM
	//   Read and unpack ADC OUTPUT DATA WORD
int r_data;

interrupt void c_int10()
{
   long  n_ch;

    r_data = _READ_SIO_DATA;
	n_ch   = ((r_data>>TSBDAS_7890_ADCODW_ADC_CHAN_SHIFT) & TSBDAS_7890_ADCODW_ADC_CHAN_BITMASK);

		// save ADC data in DATA array in particular ADC# subarray
    adc_save_ptr[adc_data_count_max*n_ch] = ((r_data <<20 )>>20);
    if(n_ch>=max_chan)
	    {
	      adc_save_ptr++;
          if((++adc_data_count) >=adc_data_count_max)
			 { 
				adc_data_count=0;
               	adc_save_ptr=adc_data_start_ptr;
			 }
	    }
}


//-- DSP Init procedure
// This demo automatically detects C33 DSP in order to correctly
// set sampling frequency via DSP on-chip timer(s), since
// C33 clock is 150 MHz instead of 60MHz for C30/C31.

unsigned	dsp_type;

#define	C33_DSP_TYPE			0x33
#define	C30_C31_DSP_TYPE		0x30

#define	_C33_DSP				(dsp_type==C33_DSP_TYPE)
#define	_C30_C31_DSP			(dsp_type==C30_C31_DSP_TYPE)

#define	_RAM_TEST_PATTERN1	0x12345678
#define	_RAM_TEST_PATTERN2	0x87654321

void init_cpu()
 {
	unsigned	*mem;

// perform detection of C33 DSP type by means of write/read to different
// DSP on-chip memory areas:
//	- C33 DSP is detected in case 0x807ffe and 0x807fff memory locations are valid
//	- C30/C31 DSP are assumed otherwise


	// assume this is C33 DSP:
	mem = (unsigned *) (C33_ONCHIP_RAM_EADDR-1);
	mem[0]	= _RAM_TEST_PATTERN1;
	mem[1]	= _RAM_TEST_PATTERN2;
	if(	mem[0]==_RAM_TEST_PATTERN1 && mem[1]==_RAM_TEST_PATTERN2)
		{
			// success!!! This is C33 DSP in TORNADO-33/TP33/E33 board
			dsp_type = C33_DSP_TYPE;
		}
		else
		{
			// assume C30/C31 DSP otherwise
			dsp_type = C30_C31_DSP_TYPE;
		}

	// This is C30/C31 DSP in E3x/T3x/TP3x
	// (all C30/C31/C33-based boards have identical value for P-BUS COntrol Register
	*C3X_PBUS_GC	= 0x700;

    C3X_INT_EDGE;		// set falling edge IRQs for C33-based boards
//	C3X_CACHE_ENABLE;
	return;
 }


void init_sio()
{
		//-- configure C3x SIO port
	*C3X_SIO0_GC 	= 	TSBDAS_7890_C3X_SIO_GCRG_DATA;
	*C3X_SIO0_XC 	= 	TSBDAS_7890_C3X_SIO_XCRG_DATA;
	*C3X_SIO0_RC 	= 	TSBDAS_7890_C3X_SIO_RCRG_DATA;
	*C3X_SIO0_GC 	|= 	TSBDAS_7890_C3X_SIO_GCRG_GO_MASK;

		//-- release SIOX reset control and enable DCM
	_REMOVE_SIOX_RESET;
}


	//-- configure TM-0 and TM-1 for sampling frequency and
	//   scan frequency generation

#define	_TM_CONTROL_RG_DATA				0x00000301  // C/P=clock, CLKSRC=internal, FUNC=timer
#define	_TM_CONTROL_RG_RUN				0x000000c0

#define _SAMPLING_FREQ					5000		// desired sampling freq (Hz)

#define _C30_C31_TIMER_SRCCLK_FREQ		15000000	// C30/C31 TM-0/1 internal src clk (Hz)
#define _C33_TIMER_SRCCLK_FREQ			37500000	// C33 TM-0/1 internal src clk (Hz)

void set_fs()
{
		//-- configure TM-0 for sampling frequency
	*C3X_TM0_GC 	= _TM_CONTROL_RG_DATA;	
	*C3X_TM0_CNT 	= 0;	
	*C3X_TM0_PER  	= ((_C33_DSP? _C33_TIMER_SRCCLK_FREQ:_C30_C31_TIMER_SRCCLK_FREQ)/(2*_SAMPLING_FREQ));	

		//-- now start TM-0 timer
	*C3X_TM0_GC |= _TM_CONTROL_RG_RUN;
}



#define	_ADDA_CHANNELS	TSBDAS_7890_DAC_CHANNELS

#define	_EPOCH_LENGTH	20

#define	TWO_PI			(2.0*3.1415926)

	//-- DACs data arrays
int data[_ADDA_CHANNELS*_EPOCH_LENGTH];

void main()
{  int i,n;

	init_cpu();
	init_sio();
	set_fs();	// configure TM-0 for sampling freq


	adc_data_start_ptr	= data;
    adc_save_ptr		= data;
	dac_data_start_ptr	= data;
    dac_get_ptr			= data;

    dac_data_count_max	= _EPOCH_LENGTH;
    adc_data_count_max	= _EPOCH_LENGTH;
	dac_data_count		= 0;
	adc_data_count		= 0;

	chan_count			= 0;
	max_chan			= _ADDA_CHANNELS-1 ;


		// enable interrupts and finally start AD/DA acquisition...
    C3X_XINT_CLEAR(0);
    C3X_XINT_ENABLE(0);

	_CLEAR_SIO_RINT_;
	_ENABLE_SIO_RINT_;

    C3X_INT_ENABLE;

	while(1);	// loop forever

}

