/********************************************************************
   T/SBDAS-7890/8420 Demo Utility for C6x. rev.1A
   (C) MicroLAB Systems, 1999-2000


   Description:
   ------------
	1. This demo provides eight pass-through AD/DA channels,
	   i.e. output of ADC[i] is loaded to DAC[i].
	2. T/SBDAS-7890/8420 must be configured in SYNCHRONOUS mode.
        3. DSP on-chip timer TM-0 is used for sampling frequency generation.

   TORNADO platform:
   -----------------
        This demo compiles for TORNADO-6x, TORNADO-P6x and TORNADO-E6x
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. MIRAGE JTAG emulator and TI CCS tools are
	required to recompile this demo and to upload code to the
	DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and is provided as it is
	without	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration jumpers:
          - SIOX IRQ-0 must be set on T/SBDAS-7890/8420
	  - on-board 10MHz synchro-clock is used
	  - SYNCHRONOUS mode is selected
	  - SIOX-Bus Device ID is '000'
	  - C6x McBSP-0 port is used (change this via _C6X_MCBSP_ def)

***********************************************************************/

#define	_E6X_		1
#define	_T6X_		2
#define	_TP6X_		3

#define	_C6X_MCBSP0_	0
#define	_C6X_MCBSP1_	1

//~~~~~~~~~ TORNADO and C6x McBSP port selectors for compilation ~~~~~~~
//!!!! modify this def in order to compile for specific TORNADO platform
#define	_TORNADO_	_E6X_

//!!!! modify this def in order to compile for McBSP-0 or McBSP-1 port
#define	_C6X_MCBSP_	_C6X_MCBSP0_
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define	_E6X_SELECTED	(_TORNADO_==_E6X_)
#define	_T6X_SELECTED	(_TORNADO_==_T6X_)
#define	_TP6X_SELECTED	(_TORNADO_==_TP6X_)


#define	_C6X_MCBSP0_SELECTED	(_C6X_MCBSP_==_C6X_MCBSP0_)
#define	_C6X_MCBSP1_SELECTED	(_C6X_MCBSP_==_C6X_MCBSP1_)


#include	"math.h"
#include	"c6x_def.h"
#include	"s7890_d.h"

//------- macros for McBSP port access ----------------------
#if(_C6X_MCBSP0_SELECTED)	// define for C6x McBSP-0 port
#define	_WAIT_SIO_TRDY		while(!((*C6X_MCBSP0_SPCR) & 0x20000))
#define	_WAIT_SIO_RRDY		while(!((*C6X_MCBSP0_SPCR) & 0x2))
#define	_WRITE_SIO_DATA		(*C6X_MCBSP0_DXR)
#define _READ_SIO_DATA		(*C6X_MCBSP0_DRR)
#endif

#if(_C6X_MCBSP1_SELECTED)	// define for C6x McBSP-1 port
#define	_WAIT_SIO_TRDY		while(!((*C6X_MCBSP1_SPCR) & 0x20000))
#define	_WAIT_SIO_RRDY		while(!((*C6X_MCBSP1_SPCR) & 0x2))
#define	_WRITE_SIO_DATA		(*C6X_MCBSP1_DXR)
#define _READ_SIO_DATA		(*C6X_MCBSP1_DRR)
#endif


//------ TORNADO platform specific fucntions --------------------
#if(_E6X_SELECTED)
#define _REMOVE_SIOX_RESET	TE6X_set_pxsx_run(0x3)
#include	"te6x_dsp.h"
#endif

#if(_T6X_SELECTED)
#define _REMOVE_SIOX_RESET	T6X_set_pxsx_run(0x7)
#include	"t6x_dsp.h"
#endif

#if(_TP6X_SELECTED)
#define _REMOVE_SIOX_RESET	TP6X_set_pxsx_run(0x7)
#include	"tp6x_dsp.h"
#endif
//--------------------------------------------------------------


#define	SXBUS_DEV_ID		0


		/* X/R interrupt handler parameters	*/

int	*dac_data_start_ptr,
        *dac_get_ptr,
	dac_data_count,
        dac_data_count_max;

int	*adc_save_ptr,
	*adc_data_start_ptr,
	adc_data_count,
	adc_data_count_max ;



	//-- F-scan interrupt fm T/SBDAS-7890/8420 DCM.
	//   This IRQ is generated via SIOX IRQ-0 interrupt request
	//   and is routed to EXT_INT4 C6x DSP interrupt request.
	//   Writes COMMAND DATA WORD to DCM.

unsigned chan_count = 0, max_chan=0;

interrupt void xmit_int()
{
	//   Create COMMAND DATA WORD.
	//   get DAC data fm particular subarray of DATA array,
	//   which is filled in with real-time ADC data.

	_WRITE_SIO_DATA =(   (SXBUS_DEV_ID << TSBDAS_7890_CDW_DEV_ID_SHIFT)
			   | (chan_count << TSBDAS_7890_CDW_DAC_CHAN_SHIFT)
			   | (chan_count << TSBDAS_7890_CDW_ADC_CHAN_SHIFT )
			   | ((chan_count>=max_chan) ?  TSBDAS_7890_CDW_EOS:TSBDAS_7890_CDW_NO_EOS )
			   | (dac_get_ptr[dac_data_count_max * chan_count] & TSBDAS_7890_CDW_DAC_DATA_BITMASK )
			 )  ;

	if( ++chan_count > max_chan)
	   {
	     dac_get_ptr++;
	     chan_count=0;
	     if(++dac_data_count >=dac_data_count_max)
	       {       dac_data_count=0;
	               dac_get_ptr=dac_data_start_ptr;
	       }
	   }

}


	//-- Receiver interrupt fm T/SBDAS-7890/8420 DCM
	//   Read and unpack ADC OUTPUT DATA WORD
unsigned r_data;

interrupt void recv_int()
{
   long  n_ch;

     	r_data= _READ_SIO_DATA;
	n_ch=((r_data>>TSBDAS_7890_ADCODW_ADC_CHAN_SHIFT) & TSBDAS_7890_ADCODW_ADC_CHAN_BITMASK);

		// save ADC data in DATA array in particular ADC# subarray
           adc_save_ptr[adc_data_count_max*n_ch] = ((r_data <<20 )>>20);
	   if(n_ch==max_chan)
	    {
	      adc_save_ptr++;
              if(++adc_data_count >=adc_data_count_max)
		 { adc_data_count=0;
                   adc_save_ptr=adc_data_start_ptr;
		 }
	    }
}




void init_cpu()
 {
	 // assume that C6x EMIF is initialized as default on reset
	 // Do not configure EMIF areas for portability.

	C6X_INT_CLEAR_ALL;
	C6X_INT_DISABLE_ALL;
	C6X_XINT_ALL_POLARITY_SET(C6X_XINT_POLARITY_NEG);

		// assume INT4    = XIRQ-0 SIOX interrupt
		// assign INT5    = McBSP-0 Receiver interrupt
	C6X_INTMUX_SET(5,C6X_INTSEL_MCBSP0_RINT);
 }

void init_sio()
{
		//-- configure C6x McBSP port

#if(_C6X_MCBSP0_SELECTED)	// define for C6x McBSP-0 port
	*C6X_MCBSP0_SPCR = TSBDAS_7890_C6X_MCBSP_SPCR_DATA;
	*C6X_MCBSP0_PCR  = TSBDAS_7890_C6X_MCBSP_PCR_DATA;
	*C6X_MCBSP0_XCR  = TSBDAS_7890_C6X_MCBSP_XCR_DATA;
	*C6X_MCBSP0_RCR  = TSBDAS_7890_C6X_MCBSP_RCR_DATA;
	*C6X_MCBSP0_SRGR = TSBDAS_7890_C6X_MCBSP_SRGR_DATA;
	*C6X_MCBSP0_SPCR |= TSBDAS_7890_C6X_MCBSP_SPCR_GO_MASK;
#endif

#if(_C6X_MCBSP1_SELECTED)	// define for C6x McBSP-1 port
	*C6X_MCBSP1_SPCR = TSBDAS_7890_C6X_MCBSP_SPCR_DATA;
	*C6X_MCBSP1_PCR  = TSBDAS_7890_C6X_MCBSP_PCR_DATA;
	*C6X_MCBSP1_XCR  = TSBDAS_7890_C6X_MCBSP_XCR_DATA;
	*C6X_MCBSP1_RCR  = TSBDAS_7890_C6X_MCBSP_RCR_DATA;
	*C6X_MCBSP1_SRGR = TSBDAS_7890_C6X_MCBSP_SRGR_DATA;
	*C6X_MCBSP1_SPCR |= TSBDAS_7890_C6X_MCBSP_SPCR_GO_MASK;
#endif

		//-- release SIOX reset control and enable DCM
	_REMOVE_SIOX_RESET;
}


	//-- configure TM-0 and TM-1 for sampling frequency and
	//   scan frequency generation

#define	_TM_CONTROL_RG_DATA	0x00000301      // C/P=clock, CLKSRC=internal, FUNC=timer
#define	_TM_CONTROL_RG_RUN	0x000000c0

#define _SAMPLING_FREQ		10000		// sampling freq (Hz)

#define _C6X_TIMER_SRCCLK_FREQ	50000000	// TM-0/1 internal src clk (Hz)
						// (for C6201 @ 200MHz)

void set_fs()
{
		//-- configure TM-0 for sampling frequency
	*C6X_TM0_CONTROL = _TM_CONTROL_RG_DATA;	
	*C6X_TM0_COUNTER = 0;	
	*C6X_TM0_PERIOD  = (_C6X_TIMER_SRCCLK_FREQ/(2*_SAMPLING_FREQ));	

		//-- now start timer
	*C6X_TM0_CONTROL |= _TM_CONTROL_RG_RUN;

}


#define	_DAC_CHANNELS	TSBDAS_7890_DAC_CHANNELS
#define	_ADC_CHANNELS	TSBDAS_7890_ADC_CHANNELS

#define	_EPOCH_LENGTH	20

	//-- DACs data arrays
int data[_DAC_CHANNELS*_EPOCH_LENGTH];

void main()
{  int i,n;

	init_cpu();
	init_sio();
	set_fs();	// configure TM-0 for sampling freq


	adc_data_start_ptr	= data;
        adc_save_ptr		= data;
	dac_data_start_ptr	= data;
        dac_get_ptr		= data;

        dac_data_count_max	= _EPOCH_LENGTH;
        adc_data_count_max	= _EPOCH_LENGTH;
	dac_data_count		= 0;
	adc_data_count		= 0;

	chan_count		= 0;
	max_chan		= (TSBDAS_7890_ADC_CHANNELS-1) ;


		// enable interrupts and finally start AD/DA acquisition...
	C6X_INT_CLEAR(4);	
	C6X_INT_CLEAR(5);
	C6X_INT_ENABLE(4);
	C6X_INT_ENABLE(5);
	C6X_GINT_ENABLE;


	while(1);	// loop forever

}

